@@ -3,8 +3,8 @@
 # Author          : Johan Vromans
 # Created On      : Sun Mar 10 18:02:02 2024
 # Last Modified By: 
-# Last Modified On: Wed May 15 12:45:20 2024
-# Update Count    : 93
+# Last Modified On: Tue May 28 20:56:30 2024
+# Update Count    : 121
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -25,15 +25,19 @@ use Getopt::Long 2.13;
 # Command line options.
 my $mode = "rrjson";
 my $execute;			# direct JSON from command line
+my $schema;			# schema (optional)
 
 # Parser options.
 my $strict;
 my $pretty = 1;
+my $croak_on_error;
+my $extra_tokens_ok;
+
+# Extension properties.
+my $order;
 my $prp;
 my $combined_keys;
 my $implied_outer_hash;
-my $croak_on_error;
-my $extra_tokens_ok;
 
 my $verbose = 1;		# verbose processing
 
@@ -48,10 +52,38 @@ my $test = 0;			# test mode.
 app_options();
 
 # Post-processing.
+
+# Default is non-strict.
+$strict //= 0;
+if ( $strict ) {
+    # No extensions.
+    $order                = 0;
+    $prp                  = 0;
+    $combined_keys        = 0;
+    $implied_outer_hash   = 0;
+}
+else {
+    # Default is all extensions.
+    $order              //= 1;
+    $prp                //= 1;
+    $combined_keys      //= 1;
+    $implied_outer_hash //= 1;
+}
+
 $trace |= ($debug || $test);
 
 ################ Presets ################
 
+# For conditional filling of hashes.
+sub maybe ( $key, $value, @rest ) {
+    if (defined $key and defined $value) {
+	return ( $key, $value, @rest );
+    }
+    else {
+	( defined($key) || @rest ) ? @rest : ();
+    }
+}
+
 ################ The Process ################
 
 use FindBin;
@@ -62,18 +94,31 @@ use Encode qw(decode_utf8);
 binmode STDOUT => ':utf8';
 binmode STDERR => ':utf8';
 
+if ( $schema ) {
+    my $parser = JSON::Relaxed::Parser->new( strict => 0 );
+    my $data = loadlines( $schema, { split => 0 } );
+    $data = $parser->decode($data);
+    warn("Schema $schema loaded\n") if $verbose;
+    $schema = $data;
+}
+
 my $parser = JSON::Relaxed::Parser->new
-  ( defined($strict) ? ( strict => $strict ) : (),
-    defined($prp) ? ( prp => $prp ) : (),
-    defined($combined_keys) ? ( combined_keys => $combined_keys ) : (),
-    defined($implied_outer_hash) ? ( implied_outer_hash => $implied_outer_hash ) : (),
-    defined($croak_on_error) ? ( croak_on_error => $croak_on_error ) : (),
-    defined($extra_tokens_ok) ? ( extra_tokens_ok => $extra_tokens_ok ) : (),
-    defined($prp) ? ( prp => $prp ) : (),
-    defined($pretty) ? ( pretty => $pretty ) : (),
-    booleans => 1,				# force default
+  ( booleans		  => 1,	# force default
+    strict	          => $strict,
+    prp		          => $prp,
+    combined_keys	  => $combined_keys,
+    implied_outer_hash    => $implied_outer_hash,
+    prp		          => $prp,
+    pretty	          => $pretty,
+    key_order	          => $order && $mode !~ /^json/,
+    maybe croak_on_error  => $croak_on_error,
+    maybe extra_tokens_ok => $extra_tokens_ok,
     );
 
+if ( $mode eq "dumper" ) {
+    $parser->booleans = [0,1];
+}
+
 if ( $verbose > 1 ) {
     my @opts;
     for ( qw( strict pretty prp combined_keys implied_outer_hash croak_on_error extra_tokens_ok booleans ) ) {
@@ -98,6 +143,8 @@ for my $file ( @ARGV ) {
     }
 
     my $data;
+
+    # For debugging/development.
     if ( $pretoks || $tokens ) {
 	$parser->croak_on_error = 0;
 	$parser->data = $json;
@@ -113,6 +160,8 @@ for my $file ( @ARGV ) {
 	}
 	$data = $parser->structure unless $parser->is_error;
     }
+
+    # Normal call.
     else {
 	$data = $parser->decode($json);
     }
@@ -128,7 +177,13 @@ for my $file ( @ARGV ) {
     }
 
     elsif ( $mode eq "rrjson" ) {
-	print $parser->encode( data => $data );
+	print $parser->encode( data => $data,
+			       maybe schema => $schema );
+	print "\n" unless $pretty;
+    }
+    elsif ( $mode eq "rjson" ) {
+	print $parser->encode( data => $data, strict => 1,
+			       maybe schema => $schema );
 	print "\n" unless $pretty;
     }
     elsif ( $mode eq "json_xs" ) {
@@ -160,15 +215,16 @@ sub dumper($data, %opts) {
     }
 
     elsif ( $mode eq "dumper" ) {
+	require Data::Dumper;
 	local $Data::Dumper::Sortkeys  = 1;
 	local $Data::Dumper::Indent    = 1;
 	local $Data::Dumper::Quotekeys = 0;
 	local $Data::Dumper::Deparse   = 1;
+	local $Data::Dumper::Purity    = 1;
 	local $Data::Dumper::Terse     = 1;
 	local $Data::Dumper::Trailingcomma = 1;
 	local $Data::Dumper::Useperl = 1;
 	local $Data::Dumper::Useqq     = 0; # I want unicode visible
-	require Data::Dumper;
 	print( Data::Dumper->Dump( [$data] ) );
     }
 }
@@ -181,29 +237,30 @@ sub app_options() {
 
     # Process options, if any.
     if ( !GetOptions(
-		     'rrjson'		    => sub { $mode = "rrjson" },
-		     'json|json_pp'	    => sub { $mode = "json" },
-		     'json_xs'		    => sub { $mode = "json_xs" },
-		     'dump'		    => sub { $mode = "dump" },
-		     'dumper'		    => sub { $mode = "dumper" },
-		     'execute|e'	    => \$execute,
-		     'strict!'		    => \$strict,
-		     'prp!'		    => \$prp,
-		     'combined_keys!'	    => \$combined_keys,
-		     'implied_outer_hash!'  => \$implied_outer_hash,
-		     'croak_on_error!'	    => \$croak_on_error,
-		     'extra_tokens_ok!'	    => \$extra_tokens_ok,
-		     'pretty!'		    => \$pretty,
-		     'pretoks+'		    => \$pretoks,
-		     'tokens+'		    => \$tokens,
-		     'ident'		    => \$ident,
-		     'verbose+'		    => \$verbose,
-		     'quiet'		    => sub { $verbose = 0 },
-		     'trace'		    => \$trace,
-		     'help|?'		    => \$help,
-		     'debug'		    => \$debug,
-		    ) or $help)
-    {
+     'schema=s'		    => \$schema,
+     'rrjson'		    => sub { $mode = "rrjson"  },
+     'rjson'		    => sub { $mode = "rjson"   },
+     'json|json_pp'	    => sub { $mode = "json"    },
+     'json_xs'		    => sub { $mode = "json_xs" },
+     'dump'		    => sub { $mode = "dump"    },
+     'dumper'		    => sub { $mode = "dumper"  },
+     'execute|e'	    => \$execute,
+     'strict!'		    => \$strict,
+     'prp!'		    => \$prp,
+     'combined_keys!'	    => \$combined_keys,
+     'implied_outer_hash!'  => \$implied_outer_hash,
+     'croak_on_error!'	    => \$croak_on_error,
+     'extra_tokens_ok!'	    => \$extra_tokens_ok,
+     'pretty!'		    => \$pretty,
+     'order!'		    => \$order,
+     'pretoks+'		    => \$pretoks,
+     'tokens+'		    => \$tokens,
+     'ident'		    => \$ident,
+     'verbose+'		    => \$verbose,
+     'quiet'		    => sub { $verbose = 0 },
+     'trace'		    => \$trace,
+     'help|?'		    => \$help,
+     'debug'		    => \$debug ) or $help) {
 	app_usage(2);
     }
     app_ident() if $ident;
@@ -221,19 +278,18 @@ sub app_usage( $exit ) {
 Usage: $0 [options] [file ...]
   Inputs
    --execute -e		args are JSON, not filenames
+   --schema=XXX		optional JSON schema
   Output modes
    --rrjson		pretty printed RRJSON output (default)
+   --rjson		pretty printed RJSON output
    --json		JSON output (default)
    --json_xs		JSON_XS output
    --no-pretty		compact (non-pretty) output
+   --order		retain order of hash keys
    --dump		dump structure (Data::Printer)
    --dumper		dump structure (Data::Dumper)
   Parser options
    --strict		see the docs
-   --prp                see the docs
-   --implied_outer_hash see the docs
-   --croak_on_error     see the docs
-   --extra_tokens_ok    see the docs
   Miscellaneous
    --ident		shows identification
    --help		shows a brief help message and exits
